/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 *  Author: Roman Khler <roman.koehler@aizo.com>
 *  Complete rewrite by: Andreas Brauchli <andreas.brauchli@aizo.com>
 */

var LOGFILE_NAME = 'timed-events.log';
var LOG = new Logger(LOGFILE_NAME);
var WEEK = ['MO','TU','WE','TH','FR','SA','SU'];

function pad(c) {
    var i = parseInt(c, 10);
    return (i < 10 ? '0' + i : c.toString());
}

function getDayIdx(d) {
    var i = 0;
    for (; i < 7; ++i) {
        if (d == WEEK[i]) {
            return i;
        }
    }
    return -1;
}

/**
 * Checks if the system version matches at least a given version
 * @param version Version to check against (in the format X.Y.Z)
 * @return true if the system version is at least the passed version,
     false otherwise, including the case where the version node doesn't exist
 */
function requireSystemVersion(version) {
    var ndSysVersion = Property.getNode('/system/version/version');
    if (ndSysVersion) {
        var sysVersion = ndSysVersion.getValue().split('.');
        var reqVersion = version.split('.');
        var len = sysVersion.length < reqVersion.length ? sysVersion.length : reqVersion.length;
        var i = 0;
        for (; i < len; ++i) {
            var sys = parseInt(sysVersion[i], 10);
            var req = parseInt(reqVersion[i], 10);
            if (req < sys || (i === len - 1 && req === sys)) {
                return true;
            }
            if (req > sys) {
                return false;
            }
        }
    }
    return false;
}

function startup() {
    // Prepare app
    Property.load();
    LOG.logln('timed-events running');

    var ndVersion = Property.getNode('version');
    if (ndVersion) {
        var version = ndVersion.getValue().split('.');
        if (version.length > 1 && version[0] == '0' && parseInt(version[1], 10) < 9) {
            convertOldEvents();
        }
    }
    // requeue events;
    var ndEvents = Property.getNode('events');
    if (ndEvents) {
        var events = ndEvents.getChildren();
        var i = 0;
        for (; i < events.length; ++i) {
            requeueEvent(events[i]);
        }

    } else {
        createNode('events');
    }

    //set version number
    Property.setProperty('version', '0.9.7');
    Property.setFlag('version', 'ARCHIVE', true);
    Property.store();
}

/**
 * A function that converts stored timed events from the old to the new version
 */
function convertOldEvents() {
    LOG.logln('Converting events from old timed events app');
    Property.load();
    var ndEvents = Property.getNode('events');
    var events = ndEvents.getChildren();
    var i = 0;
    for (; i < events.length; ++i) {
        var ndOldParams = events[i].getChild('params');
        if (!ndOldParams) {
            LOG.logln('Warning: no old params found to convert on event ' + events[i].name);
            continue;
        }
        // LOG.logln('Debug: old params: ' + ndOldParams.getValue());
        var oldParams = JSON.parse(ndOldParams.getValue());
        // LOG.logln('Debug: old recurrence: ' + decodeURIComponent(oldParams.recurrence));
        var oldRecurrence = JSON.parse(decodeURIComponent(oldParams.recurrence));
        // LOG.logln('Debug: old actions: ' + decodeURIComponent(oldParams.actions));
        var oldActions = JSON.parse(decodeURIComponent(oldParams.actions));

        //
        // validity checks
        //

        if (!oldRecurrence
            || !oldActions
            || !oldActions.actions
            || !(oldActions.actions.length > 0)
        ) {
            // delete invalid events
            LOG.logln('Deleting event '+ events[i].name +': missing recurrence or actions');
            deleteOldEvent(events[i]);
            continue;
        }
        oldActions = oldActions.actions;

        if (! oldRecurrence.triggerTime
            || (oldRecurrence.recurrence == 'irregular'
                && (!oldRecurrence.dates
                    || oldRecurrence.dates.length == 0))
            || (oldRecurrence.recurrence == 'regular_wk'
                && (!oldRecurrence.weekdays
                    || oldRecurrence.weekdays.length == 0))
        ) {
            LOG.logln('Deleting event '+ events[i].name +': missing time, dates or week days');
            deleteOldEvent(events[i]);
            continue;
        }

        //
        // Conversions
        //

        // new params object
        var params = {
            name: oldParams.name
        };

        // disabled event
        if (oldParams.Disabled === true) {
            params.disabled = true;
        }

        // Convert time from UTC to local time
        // (assuming correctly set timezone on the dSS)
        var utc = new Date();
        var time = oldRecurrence.triggerTime.split(':');
        utc.setUTCHours(parseInt(time[0], 10));
        utc.setUTCMinutes(parseInt(time[1], 10));
        utc.setUTCSeconds(parseInt(time[2], 10));
        params.time = // utc.format('H:i:s');
                    pad(utc.getHours())
                    + ':' + pad(utc.getMinutes())
                    + ':' + pad(utc.getSeconds());

        // recurrence period
        if (oldRecurrence.recurrence == 'regular_wk') {
            params.recurring = true;
        }

        if (params.recurring) {
            // weekdays
            params.recurrence = [];
            var j = 0;
            for (; j < oldRecurrence.weekdays.length; ++j) {
                var idx = getDayIdx(oldRecurrence.weekdays[j]);
                if (idx !== -1) {
                    params.recurrence.push(idx);
                } else {
                    LOG.logln('Skipping invalid weekday: ' + oldRecurrence.weekdays[j]);
                    continue;
                }
            }
            if (params.recurrence.length == 0) {
                LOG.logln('Deleting event '+ events[i].name +': no valid weekdays');
                deleteOldEvent(events[i]);
                continue;
            }

        } else {
            // reverse the dates from 13-01-2001 to 2001-01-13
            var dates = oldRecurrence.dates;
            var d = 0;
            params.recurrence = [];
            for (; d < dates.length; ++d) {
                var date = dates[d].split('-');
                if (date.length != 3
                    || parseInt(date[1], 10) > 12) {
                    LOG.logln('Skipping invalid date');
                    continue;
                }
                params.recurrence.push(
                    pad(parseInt(date[2], 10)) + '-'
                    + pad(parseInt(date[1], 10)) + '-'
                    + pad(parseInt(date[0], 10))
                );
            }
            if (params.recurrence.length == 0) {
                LOG.logln('Deleting event '+ events[i].name +': no valid dates');
                deleteOldEvent(events[i]);
                continue;
            }
        }

        // Convert old actions
        params.actions = [];
        var a = 0;
        for (; a < oldActions.length; ++a) {
            var p = {};
            if (oldActions[a].Zone !== undefined) {
                p.zone = parseInt(oldActions[a].Zone, 10);
            }
            if (oldActions[a].Group !== undefined) {
                p.group = oldActions[a].Group === '' ?
                    0 : parseInt(oldActions[a].Group, 10);
            }
            if (oldActions[a].Scene !== undefined) {
                p.scene = parseInt(oldActions[a].Scene, 10);
            }
            if (oldActions[a].Output !== undefined) {
                p.outputValue = parseInt(oldActions[a].Output, 10);
            }
            if (oldActions[a].Device) {
                p.device = oldActions[a].Device;
            }
            if (oldActions[a].Blink) {
                p.blink = true;
            }
            params.actions.push(p);
        }

        // store converted event
        deleteOldEvent(events[i]);
        addEvent(params);
        LOG.logln('Converted event '+ params.name);
    }
    Property.store();
} // convertOldEvents

/**
 * Execute actions delayed one by one
 * @param actions {Array} Actions to execute (in the order of the array)
 * @param time {Number} Miliseconds to wait between actions
 * @param callback {Function} (optional) callback to execute after the last action execution
 */
function executeActions(actions, time, callback) {
    var action = actions.shift();

    var target = null;
    if (action.group != null) {
        LOG.logln('target zone ' + action.zone + ' group ' + action.group);
        target = getDevices().byZone(action.zone).byGroup(action.group);

    } else if (action.device != null) {
        LOG.logln('target device ' + action.device);
        target = getDevices().byDSID(action.device);
    }

    if (target) {
        if (action.scene !== undefined) {
            LOG.logln('Exec: calling scene ' + action.scene);
            target.callScene(parseInt(action.scene, 10));
        }
        else if (action.blink) {
            LOG.logln('Exec: blinking');
            target.blink();
        }
        else if (action.outputValue !== undefined) {
            LOG.logln('Exec: setting ouput ' + action.outputValue);
            target.setValue(parseInt(action.outputValue, 10));
        }
    } else {
        LOG.logln('Warning: empty target');
    }

    if (actions.length > 0) {
        LOG.logln('Scheduling next '+ actions.length +' params in '+ time +'ms');
        var o = function () { executeActions(actions, time, callback); };
        setTimeout(time, o);
    } else {
        callback();
    }
} // executeActions

/**
 * Parses the recurrence object and generates an array of strings, each
 * containing a valid recurrence rule.
 * The dSS only accepts iCal dates if one provides both rrule and start time
 * @param params {Object} The event's parameters object
 * @return {Array} An array of rrule/starting time pair objects
 */
function generateIcalRules(params) {
    var rules = [];
    var time = params.time.split(':');
    var d = new Date();
    var i;
    var use_utc_time = !requireSystemVersion('1.0.5'); // use compatibility mode with old dSS versions (< 1.0.5)
    d.setHours(parseInt(time[0], 10));
    d.setMinutes(parseInt(time[1], 10));
    d.setSeconds(parseInt(time[2], 10));

    var timestring = '';
    if (use_utc_time) {
        // run in compatibility mode
        timestring = pad(d.getUTCHours())
            + pad(d.getUTCMinutes())
            + pad(d.getUTCSeconds())
            + 'Z';
    } else {
        // use local time
        timestring = pad(d.getHours())
            + pad(d.getMinutes())
            + pad(d.getSeconds());
        // LOG.logln('Debug: event scheduled in local time for ' + d);
    }

    var rrule = null;
    var iCalStartTime = null;
    if (params.recurring) {
        iCalStartTime =
            d.getFullYear()
            + pad(d.getMonth() + 1) // goes from 0..11
            + pad(d.getDate())
            + 'T'
            + timestring;

        var weekdays = [];
        for (i = 0; i < params.recurrence.length; ++i) {
            weekdays.push(WEEK[params.recurrence[i]]);
        }
        rrule = 'FREQ=WEEKLY;BYDAY=' + weekdays;

        rules.push({
            iCalRRule: rrule,
            iCalStartTTime: iCalStartTime
        });

    } else {
        rrule = 'FREQ=DAILY;COUNT=1';
        for (i = 0; i < params.recurrence.length; ++i) {
            iCalStartTime = params.recurrence[i].replace(/-/g, '') + 'T' + timestring;
            rules.push({
                iCalRRule: rrule,
                iCalStartTTime: iCalStartTime
            });
        }
    }
    // LOG.logln('Debug: rrule: `'+ rrule +'\', start time: ' + iCalStartTime);

    return rules;
}

/**
 * Remove all queued occurences of this event
 * @param ndEvent The event node of the event to reqeue
 */
function dequeueEvent(ndEvent) {
    var ndQueue = ndEvent.getChild('queueIDs');
    if (!ndQueue) {
        return;
    }
    var queue = ndQueue.getChildren();
    var i = 0;
    for (; i < queue.length; ++i) {
        var propNode = Property.getNode(
            '/system/EventInterpreter/ScheduledEvents/' + queue[i].getValue()
        );
        if (propNode) {
            propNode.getParent().removeChild(propNode);
        }
        ndQueue.removeChild(queue[i]);
    }
}
/**
 * Queue an event
 * @param ndEvent The event node of the event to reqeue
 * @param params (optional) pre-loaded params of the node
 */
function queueEvent(ndEvent, params) {
    var qIdPath = 'events/' + ndEvent.name + '/queueIDs';
    var ndQueue = ndEvent.getChild('queueIDs') || createNode(qIdPath); // just here to ensure the node is created
    var i;

    // raise the event and store the queue id
    if (!params) {
        params = loadParams(ndEvent);
    }
    if (params.disabled) {
        return;
    }
    LOG.logln('Debug: queueing events for timer ' + params.name);

    var ical = generateIcalRules(params);
    for (i = 0; i < ical.length; ++i) {
        var timerEvent = new TimedICalEvent(
            'timed-events',
            ical[i].iCalStartTTime,
            ical[i].iCalRRule,
            {
                event_name: params.name,
                action_type: 'execute'
            }
        );
        Property.setProperty(qIdPath +'/'+ i, timerEvent.raise());
    }
}

/**
 * Check if all occurences of an event are in the queue, else requeue them
 * @param ndEvent The event node of the event to reqeue
 * @param params (optional) pre-loaded params of the node
 */
function requeueEvent(ndEvent, params) {
    var ndQueue = ndEvent.getChild('queueIDs');
    var i, queue, propNode;

    if (!ndQueue) {
        queueEvent(ndEvent, params);

    } else {
        queue = ndQueue.getChildren();
        if (queue.length == 0) {
            queueEvent(ndEvent, params);

        } else {
            for (i = 0; i < queue.length; ++i) {
                propNode = Property.getNode(
                    '/system/EventInterpreter/ScheduledEvents/' + queue[i].getValue()
                );
                if (!propNode) {
                    // if any node doesn't exists, delete and requeue them all
                    dequeueEvent(ndEvent);
                    queueEvent(ndEvent, params);
                    break;
                }
            }
        }
    }
}

/**
 * Fetches the parameters of this event from the property tree.
 * @param eventNode {Object} Property tree node of the event
 * @return {Object} Object containing the read out values of the event
 */
function loadParams(eventNode) {
    LOG.logln('Loading params for node: ' + eventNode.name);
    var name = eventNode.getChild('name');
    if (!name) {
        LOG.logln('Error: empty or invalid node');
        return;
    }

    var params = {
        name: name.getValue(),
        time: eventNode.getChild('time').getValue(),
        actions: [],
        recurrence: []
    };
    var ndDisabled = eventNode.getChild('disabled');
    params.disabled = (ndDisabled ? !! ndDisabled.getValue() : false);

    // LOG.logln('Debug: Loading actions');
    var ndActions = eventNode.getChild('actions');
    var loadValue = function(action, ndAction, name) {
        var c = ndAction.getChild(name);
        if (c) {
            action[name] = c.getValue();
            return true;
        }
        return false;
    };
    if (ndActions) {
        var actions = ndActions.getChildren();
        var a = 0;
        for (; a < actions.length; ++a) {
            var curAction = {};
            if (loadValue(curAction, actions[a], 'device')) {}
            else if (loadValue(curAction, actions[a], 'group') && loadValue(curAction, actions[a], 'zone')) {}
            else {
                LOG.logln('Error: event '+ params.name +' has an action with no target');
                continue;
            }
            if (loadValue(curAction, actions[a], 'blink')) {}
            else if (loadValue(curAction, actions[a], 'scene')) {}
            else if (loadValue(curAction, actions[a], 'outputValue')) {}
            else {
                LOG.logln('Error: event '+ params.name +' has an action with invalid action to take');
                continue;
            }
            params.actions.push(curAction);
            // LOG.logln('Debug: action: '+ JSON.stringify(curAction));
        }
    } else {
        LOG.logln('Error: event '+ params.name +' has no actions set');
    }

    // LOG.logln('Debug: loading recurrences');
    var ndRecurring = eventNode.getChild('recurring');
    params.recurring = (ndRecurring ? !!ndRecurring.getValue() : false);

    var ndRecurrence = eventNode.getChild('recurrence');
    if (ndRecurrence) {
        var recurrences = ndRecurrence.getChildren();
        if (params.recurring) {
            // weekdays
            var w = 0;
            for (; w < recurrences.length; ++w) {
                params.recurrence.push(recurrences[w].name);
            }
        } else {
            // dates
            var d = 0;
            for (; d < recurrences.length; ++d) {
                params.recurrence.push(recurrences[d].getValue());
            }
        }
        // LOG.logln('Debug: recurrences: '+ JSON.stringify(params.recurrence));
    } else {
        LOG.logln('Error: event '+ params.name +' has no recurrence set');
    }
    // LOG.logln('Debug: params loaded');
    return params;
}

/**
 * Updates a node if it exists or creates a new archived property node otherwise
 * @param path {String} Relative path
 * @param value {String|Boolean|Integer} Value to set
 */
function updateOrCreateNode(path, value) {
    var nd = Property.getNode(path);
    if (nd) {
        nd.setValue(value);
    } else {
        createNode(path, value);
    }
}

/**
 * Creates a new archived property node with given value
 * @param path {String} Relative path
 * @param value {String|Boolean|Integer} Value to set
 * @return {Object} The newly created node
 */
function createNode(path, value) {
    var nd = new Property(path);
    if (value !== undefined) {
        nd.setValue(value);
    }
    nd.setFlag('ARCHIVE', true);
    return nd;
}

/**
 * Stores _VALID_ parameters into the property tree.
 * Use checkParams(params) to validate them first!
 * Make sure to call Property.store() after calling this method
 * @param params {Object} Object to store
 */
function storeParams(params) {
    var eventPath = 'events/' + params.name;
    var actionPath = eventPath + '/actions';
    var i, c;

    var ndEvent = Property.getNode(eventPath) || createNode(eventPath);

    updateOrCreateNode(eventPath + '/name', params.name);
    updateOrCreateNode(eventPath + '/time', params.time);
    updateOrCreateNode(eventPath + '/disabled', !!params.disabled);

    // actions
    var ndActions = ndEvent.getChild('actions');
    if (!ndActions) {
        ndActions = createNode(actionPath);
    } else {
        // clean out actions, as indices might not match up
        for (c in ndActions.getChildren()) {
            ndActions.removeChild(c);
        }
    }
    // populate actions
    for (i = 0; i < params.actions.length; ++i) {
        var p = actionPath + '/' + i;
        createNode(p);

        if (params.actions[i].zone !== undefined) {
            createNode(p + '/zone', params.actions[i].zone);
        }
        if (params.actions[i].group !== undefined) {
            createNode(p + '/group', params.actions[i].group);
        }
        if (params.actions[i].device) {
            createNode(p + '/device', params.actions[i].device);
        }
        if (params.actions[i].blink) {
            createNode(p + '/blink', true);
        }
        if (params.actions[i].scene !== undefined) {
            createNode(p + '/scene', params.actions[i].scene);
        }
        if (params.actions[i].outputValue !== undefined) {
            createNode(p + '/outputValue', params.actions[i].outputValue);
        }
    }

    // recurrence
    updateOrCreateNode(eventPath + '/recurring', params.recurring);

    var ndRecurrence = ndEvent.getChild('recurrence')
                    || createNode(eventPath + '/recurrence');
    if (params.recurring) {
        // weed out existing weekdays
        for (c in ndRecurrence.getChildren()) {
            ndRecurrence.removeChild(c);
        }
        for (i = 0; i < params.recurrence.length; ++i) {
            var idx = params.recurrence[i];
            createNode(eventPath + '/recurrence/' + idx, WEEK[idx]);
        }

    } else {
        // weed out dates
        for (c in ndRecurrence.getChildren()) {
            ndRecurrence.removeChild(c);
        }
        for (i = 0; i < params.recurrence.length; ++i) {
            createNode(eventPath + '/recurrence/' + i, params.recurrence[i]);
        }
    }
    return ndEvent;
} // storeParams

/**
 * Checks if parameters conform to what is expected
 * @param eventParams {Object} Object to check
 * @return {Object} Conform object or null on error
 */
function checkParams(eventParams) {
    if (!eventParams.name
        || eventParams.name.length === 0
        || eventParams.name.indexOf('/') !== -1
    ) {
        LOG.logln('Error: event has no or invalid name');
        return null;
    }

    if (!eventParams.actions || eventParams.actions.length === 0) {
        LOG.logln('Error: event has no actions');
        return null;
    }

    if (!eventParams.recurrence || eventParams.recurrence.length === 0) {
        LOG.logln('Error: event has no recurrence');
        return null;
    }

    if (!eventParams.time) {
        LOG.logln('Error: event has no time set');
        return null;
    }

    var actions = [];
    var i = 0;
    for (; i < eventParams.actions.length; ++i) {
        var a = eventParams.actions[i];
        if (!a.device && isNaN(a.group)) { // isNan(undefined) === true
            LOG.logln('Error: no or invalid group or device defined as target on action ' + i);
            return null;
        }
        if (a.blink === undefined && a.scene === undefined && a.outputValue === undefined) {
            LOG.logln('Error: no action defined at index ' + i);
            return null;
        }
        if (
            (a.scene !== undefined && isNaN(a.scene))
            || (a.group !== undefined && isNaN(a.group))
            || (a.outputValue !== undefined && (
                isNaN(a.outputValue) || a.outputValue < 0 || a.outputValue > 255
            ))
        ) {
            LOG.logln('Error: invalid action defined at index ' + i);
            return null;
        }

        var action = {};
        if (a.zone !== undefined) {
            action.zone = parseInt(a.zone, 10);
        }
        if (a.group !== undefined) {
            action.group = parseInt(a.group, 10);
        }
        if (a.device !== undefined) {
            action.device = a.device;
        }
        if (a.blink) {
            action.blink = true;
        }
        if (a.scene !== undefined) {
            action.scene = parseInt(a.scene, 10);
        }
        if (a.outputValue !== undefined) {
            action.outputValue = parseInt(a.outputValue, 10);
        }
        actions.push(action);
    }

    var recurring = !!eventParams.recurring;
    var recurrence = [];
    var d;
    if (recurring) {
        for (i = 0; i < eventParams.recurrence.length; ++i) {
            d = eventParams.recurrence[i];
            if (isNaN(d) || d < 0 || d > 6) {
                LOG.logln('Warning: wrong weekday index `'+ d +'\', skipping');
                continue;
            }
            recurrence.push(d);
        }
    } else {
        for (i = 0; i < eventParams.recurrence.length; ++i) {
            d = eventParams.recurrence[i];
            var date = d.split('-');
            if (date.length !== 3 || isNaN(date[0]) || isNaN(date[1]) || isNaN(date[2])
                || date[0].length < 4 || date[1] == 0 || date[1] > 12
                || date[2] == 0 || date[2] > 31
            ) {
                LOG.logln('Warning: invalid date `'+ d +'\'');
                continue;
            }
            recurrence.push(d);
        }
    }
    if (recurrence.length == 0) {
        LOG.logln('Error: no valid recurrence');
        return null;
    }
    var t = eventParams.time.split(':');
    var inv = (t.length !== 3);
    for (i = 0; !inv && i < 3; ++i) {
        if (!isNaN(t[i])) {
            t[i] = parseInt(t[i], 10);
            inv = inv || t[i] < 0 || t[i] > 59;
        } else {
            inv = true;
        }
    }
    inv = inv || (t[0] > 23);
    if (inv) {
        LOG.logln('Error: invalid time format');
        return null;
    }
    var time = pad(t[0]) + ':' + pad(t[1]) + ':' + pad(t[2]);

    // create the valid params object
    var params = {
        name: eventParams.name,
        actions: actions,
        disabled: !!eventParams.disabled,
        recurring: recurring,
        recurrence: recurrence,
        time: time
    };

    return params;
} // checkParams

/**
 * Add a new event
 * @param params {Object} The new event is created from these parameters
 */
function addEvent(eventParams) {
    // check eventParams and set params for serialization
    var params = checkParams(eventParams);
    if (!params) {
        return;
    }

    var ndEvent = storeParams(params);

    if (!params.disabled) {
        queueEvent(ndEvent, params);
    }

    // set archive flags
    Property.setFlag('events/' + params.name, 'ARCHIVE', true);
    Property.store();
}

function executeEvent(eventNode, params) {
    if (!params.disabled) {
        LOG.logln(
            'Processing event ' + params.name
            + ' (' + params.actions.length +' actions)'
        );
        executeActions(params.actions, 500, function() {
            disableOnLastOccurence(eventNode, params);
        });
    }
}

function disableOnLastOccurence(eventNode, params) {
    // Disable irregular event if it's its last occurrence
    if (!params.recurring) {
        if (eventNode.getChild('queueIDs').getChildren().length === 1) {
            enableEvent(eventNode, params, false);

            // notify UI about changes
            var updateEvent = new Event('timed-events.notifyUi', params.name);
            updateEvent.raise();
        }
    }
}

function deleteOldEvent(eventNode) {
    deleteEvent(eventNode, { name: eventNode.name });
}

function deleteEvent(eventNode, params) {
    LOG.logln('Delete: event '+ params.name);

    dequeueEvent(eventNode);
    eventNode.getParent().removeChild(eventNode);
    Property.store();
}

/**
 * Enable or disable a timed event
 * @param enable {Boolean} True to enable, false to disable
 */
function enableEvent(eventNode, params, enable) {
    LOG.logln((enable ? 'Enabling' : 'Disabling') + ' event ' + params.name);

    params.disabled = !enable;
    updateOrCreateNode('events/' + params.name + '/disabled', !enable);
    if (enable) {
        requeueEvent(eventNode, params);
    } else {
        dequeueEvent(eventNode);
    }
    Property.store();
}

function main() {
    if (raisedEvent.name == 'running') {
        startup();
        return;
    }
    /*
    LOG.logln('Debug: timed-events main call with action type: '
                + raisedEvent.parameter.action_type
                + (raisedEvent.parameter.params === undefined ? ''
                    : ' and params: '
                    + raisedEvent.parameter.params.toString()
                )
    );
    */
    var params;
    if (raisedEvent.parameter.action_type == 'add') {
        // Add a new event
        if (raisedEvent.parameter.params === undefined) {
            LOG.logln('Error: missing event parameters');
            return;
        }
        LOG.logln('Add event with parmeters: '+ raisedEvent.parameter.params);
        params = JSON.parse(raisedEvent.parameter.params);
        addEvent(params);

    } else {
        // Operations that involve a specific event's parameters
        Property.load();
        var ndEvent = Property.getNode('events/' + raisedEvent.parameter.event_name);
        if (ndEvent) {
            params = loadParams(ndEvent);

            if (raisedEvent.parameter.action_type == 'enable') {
                enableEvent(ndEvent, params, true);
            }
            else if (raisedEvent.parameter.action_type == 'disable') {
                enableEvent(ndEvent, params, false);
            }
            else if (raisedEvent.parameter.action_type == 'delete') {
                deleteEvent(ndEvent, params);
            }
            else if (raisedEvent.parameter.action_type == 'execute') {
                executeEvent(ndEvent, params);
            }
        } else {
            LOG.logln('Error: '+ raisedEvent.parameter.event_name +' doesn\'t exist');
        }
    }
} // main

main();

